package ru.karelia.cs.petrsugen.handlers;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;



//import ru.karelia.cs.petrsugen.templates.TemplateResourceFile;
import ru.karelia.cs.petrsugen.templates.TemplateResourceFile;

import com.hp.hpl.jena.graph.Triple;
import com.hp.hpl.jena.ontology.CardinalityRestriction;
import com.hp.hpl.jena.ontology.OntClass;
import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.ontology.OntProperty;
import com.hp.hpl.jena.ontology.OntResource;
import com.hp.hpl.jena.ontology.Restriction;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.rdf.model.StmtIterator;
import com.hp.hpl.jena.util.iterator.ExtendedIterator;

import ru.karelia.cs.petrsugen.framework.CCodeModelContainer;
import ru.karelia.cs.petrsugen.util.CodeGenerationUtils;
import ru.karelia.cs.petrsugen.util.Logger;
import ru.karelia.cs.petrsugen.util.TemplateString;

/**
 *
 * 
 */
public class OntHandler {

	static String ResourcePath = "/ru/karelia/cs/petrsugen/templates/";
/////////////
   	static String OntClassDefineTemplateName = TemplateResourceFile.getTemplate(ResourcePath + "OntClassDefine");
   	static String OntPropertyDefineTemplateName = TemplateResourceFile.getTemplate(ResourcePath + "OntPropertyDefine");
/////////////	
	
	static String OntImplementationTemplateName = TemplateResourceFile.getTemplate(ResourcePath + "OntImplementation");
	static String OntClassImplementationTemplateName = TemplateResourceFile.getTemplate(ResourcePath + "OntClassImplementation");
	static String OntPropertyImplementationTemplateName = TemplateResourceFile.getTemplate(ResourcePath + "OntPropertyImplementation");
	static String OntClassSuperclassImplementationTemplateName = TemplateResourceFile.getTemplate(ResourcePath + "OntClassSuperclassImplementation");
	static String OntClassPropertiesImplementationTemplateName = TemplateResourceFile.getTemplate(ResourcePath + "OntClassPropertiesImplementation");
	static String OntPropertySubPropertyImplementationTemplateName =  TemplateResourceFile.getTemplate(ResourcePath + "OntPropertySubPropertyImplementation");
	
	static String  OntDefinitionTemplateName = TemplateResourceFile.getTemplate(ResourcePath + "OntDefinition");
    static String  OntDefinitionCFileTemplateName = TemplateResourceFile.getTemplate(ResourcePath + "OntDefinitionCFile");

	static String  OntClassDefinitionTemplateName = TemplateResourceFile.getTemplate(ResourcePath + "OntClassDefinition");
	static String  OntPropertyDefinitionTemplateName = TemplateResourceFile.getTemplate(ResourcePath + "OntPropertyDefinition");

    static String  OntClassDefinitionExternTemplateName = TemplateResourceFile.getTemplate(ResourcePath + "OntClassDefinitionExtern");
	static String  OntPropertyDefinitionExternTemplateName = TemplateResourceFile.getTemplate(ResourcePath + "OntPropertyDefinitionExtern");

	public static LinkedList<LinkedList<String>> getTripleOntology(OntModel model) {
	    	LinkedList<LinkedList<String>> triples = new LinkedList<LinkedList<String>>();
	    	if (model != null) {
	            /*
	            this.classes = new LinkedList<ClassTree>();
	            this.deserializeModel(model);
	             */
	            StmtIterator iterator = model.listStatements();
	            LinkedList<String> tripleList;
	            Triple triple = null;
	            int cntStatements = 0;
	            int cntStatementsInBaseModel = 0;
	            while (iterator.hasNext()) {
	                tripleList = new LinkedList<String>();
	                Statement statement = iterator.nextStatement();
	                cntStatements++;
	                if ( model.isInBaseModel(statement) ) {
	                    //System.out.println("Yes "+cntStatementsInBaseModel);
	                    cntStatementsInBaseModel++;
	                    triple = statement.asTriple();

	                    tripleList.add(triple.getSubject().toString());
	                    tripleList.add(triple.getPredicate().toString());
	                    tripleList.add(triple.getObject().toString());
	                    if (triple.getObject().isLiteral()) {
	                        tripleList.add("True");
	                    } else if (triple.getObject().isURI()) {
	                        tripleList.add("False");
	                    } else if (triple.getObject().isConcrete()) {
	                        tripleList.add("False");
	                    } else if (triple.getObject().isVariable()) {
	                        tripleList.add("False");
	                    } else if (triple.getObject().isBlank()) {
	                        tripleList.add("False");
	                    }
	                    triples.add(tripleList);
	                } else {
	                    //System.out.println("No "+ (cntStatements-cntStatementsInBaseModel ));
	                }
	            }
	            //System.out.println("Total statements (assertions): " + cntStatements);
	            //System.out.println("Statements in base model (assertions): " + cntStatements);
	        }
	    
	    	return triples;
	    }
	
    
    public static void handle(OntModel ontModel, CCodeModelContainer writer, Logger log) {
      
        TemplateString OntImplementation = new TemplateString(OntImplementationTemplateName);
        TemplateString OntClassImplementation = new TemplateString(OntClassImplementationTemplateName);
        TemplateString OntPropertyImplementation = new TemplateString(OntPropertyImplementationTemplateName);
    	TemplateString OntPropertySubPropertyImplementation = new TemplateString(OntPropertySubPropertyImplementationTemplateName);
        TemplateString OntClassPropertiesImplementation = new TemplateString(OntClassPropertiesImplementationTemplateName);
        TemplateString OntClassSuperclassImplementation = new TemplateString(OntClassSuperclassImplementationTemplateName);
    	
        TemplateString OntDefinition = new TemplateString(OntDefinitionTemplateName);
        TemplateString OntClassDefinition = new TemplateString(OntClassDefinitionTemplateName);
        TemplateString OntPropertyDefinition = new TemplateString(OntPropertyDefinitionTemplateName);

//////
        TemplateString OntDefinitionCFile = new TemplateString(OntDefinitionCFileTemplateName);
        TemplateString OntClassDefinitionExtern = new TemplateString(OntClassDefinitionExternTemplateName);
        TemplateString OntPropertyDefinitionExtern = new TemplateString(OntPropertyDefinitionExternTemplateName);

///////////
        TemplateString OntClassDefine = new TemplateString(OntClassDefineTemplateName);
        TemplateString OntPropertyDefine = new TemplateString(OntPropertyDefineTemplateName);
//////////        
        
        
        
        
    	String prefix="";
    	
    	OntImplementation.replaceTag("<@headerfile@>", writer.getHeaderFile());
        OntDefinitionCFile.replaceTag("<@headerfile@>", writer.getHeaderFile());
    	
    	
    	ExtendedIterator itr = ontModel.listOntProperties();
    	
    	while(itr.hasNext()){
    		OntProperty op = (OntProperty) itr.next();
    		int maxcardinality = -1;
    		int mincardinality = -1;
    		System.out.println("OP: " + op.getURI());
			if (op.isFunctionalProperty()){
    			System.out.println("FUNCTIONAL PROP "+ op.getLocalName());        			
    			maxcardinality = 1;
    		}
			
    		
    		if (op.isDatatypeProperty() || op.isObjectProperty()){
    				
                ExtendedIterator restrictions = op.listReferringRestrictions();
                while(restrictions.hasNext()){
                	
                	Restriction res = (Restriction) restrictions.next();
                	if (res.isCardinalityRestriction()){
                        CardinalityRestriction cardinality = res.asCardinalityRestriction();
                        if (cardinality.isMaxCardinalityRestriction())
                        	mincardinality = cardinality.getCardinality();
                        
                        if (cardinality.isMaxCardinalityRestriction())
                        	maxcardinality = cardinality.getCardinality();
                        
                        System.out.print("Cardinaliy ");
                	}
                	System.out.println("RESTRICTION " + res.getLocalName());
                }
    			
    			String propName = op.getLocalName();
    			
    			OntPropertyDefinitionExtern.replaceTag("<@propertyname@>", propName);
    			
//////////////////
    			OntPropertyDefinitionExtern.replaceTag("<@propertynameuppercase@>", propName.toUpperCase());
//////////////////    			
    			OntPropertyDefinitionExtern.replaceTag("<@prefix@>", prefix);
    			OntDefinition.insertIntoPlaceholder("<%properties%>", OntPropertyDefinitionExtern.getString());


                OntPropertyDefinition.replaceTag("<@propertyname@>", propName);
    			OntPropertyDefinition.replaceTag("<@propertynameuppercase@>", propName.toUpperCase());
    			OntPropertyDefinition.replaceTag("<@prefix@>", prefix);
    			OntDefinitionCFile.insertIntoPlaceholder("<%properties%>", OntPropertyDefinition.getString());
    			OntImplementation.insertIntoPlaceholder("<%propertiesdefs%>", OntPropertyDefinition.getString());

//////////////////
    			OntPropertyDefine.replaceTag("<@propertynameuppercase@>", propName.toUpperCase() + " 1"); 
    			OntDefinition.insertIntoPlaceholder("<%includeontentities%>", OntPropertyDefine.getString());
       			OntPropertyDefine.resetToInitValue();
       			
    			OntPropertyDefine.replaceTag("<@propertynameuppercase@>", propName.toUpperCase() + " 0");
    			OntDefinition.insertIntoPlaceholder("<%excludeontentities%>", OntPropertyDefine.getString());
    			OntPropertyDefine.resetToInitValue();
//////////////////
    			
    			OntPropertyDefinition.resetToInitValue();
                OntPropertyDefinitionExtern.resetToInitValue();
    			

//////////////////
    			OntPropertyImplementation.replaceTag("<@propertynameuppercase@>", propName.toUpperCase());
//////////////////    			
    			
    			if (op.isObjectProperty()) {
        			OntPropertyImplementation.replaceTag("<@proptype@>", "OBJECTPROPERTY");    			
        		} else {	
        			OntPropertyImplementation.replaceTag("<@proptype@>", "DATATYPEPROPERTY");
        		}	
    			
    			OntPropertyImplementation.replaceTag("<@propertyname@>", propName);
    			OntPropertyImplementation.replaceTag("<@prefix@>", prefix);
    			OntPropertyImplementation.replaceTag("<@maxcardinality@>", Integer.toString(maxcardinality));
    			OntPropertyImplementation.replaceTag("<@mincardinality@>", Integer.toString(mincardinality ));
    			OntPropertyImplementation.replaceTag("<@name@>", propName);
    			
    			OntPropertyImplementation.replaceTag("<@about@>", op.getURI().toString());
    			if (op.getRange()!= null){
    				OntPropertyImplementation.replaceTag("<@range@>", op.getRange().toString());
    			} else {
    				System.out.println(" OP has null Range: "+ propName);    			
    			}
    			if (op.getDomain()!= null){
    				OntPropertyImplementation.replaceTag("<@domain@>", op.getDomain().toString());
    			} else {
    				System.out.println(" OP has null Domain: "+ propName);
    			
    			}
    			if (op.isInverseFunctionalProperty()) {
    				OntPropertyImplementation.replaceTag("<@inversfunctionalproperty@>", "\""+op.getRange().toString()+"\"");	
       			
        		} else {
        			OntPropertyImplementation.replaceTag("<@inversfunctionalproperty@>", "NULL");
        			
        		}
        		
    			if (op.isObjectProperty()){
    				OntPropertyImplementation.replaceTag("<@type@>", "OBJECTPROPERTY");
    				OntPropertyImplementation.replaceTag("<@set@>", "op_property_set");
    				OntPropertyImplementation.replaceTag("<@xsdtype@>", "NONE");
    			} else {
    				OntPropertyImplementation.replaceTag("<@type@>", "DATATYPEPROPERTY");
    				OntPropertyImplementation.replaceTag("<@set@>", "dp_property_set");
    				String dpType = CodeGenerationUtils.removeUri(op.getRange().toString());
    				OntPropertyImplementation.replaceTag("<@xsdtype@>", "XSD"+dpType.toUpperCase());
    			}
    			ExtendedIterator sitr = op.listSuperProperties();
    			while(sitr.hasNext()){
    				
    				OntProperty sup = (OntProperty) sitr.next();
    				if (!sup.getLocalName().equals(op.getLocalName())){
    					OntPropertySubPropertyImplementation.replaceTag("<@prefix@>", prefix);
    					OntPropertySubPropertyImplementation.replaceTag("<@propertyname@>", propName);
    					OntPropertySubPropertyImplementation.replaceTag("<@property@>", sup.getLocalName());
    					
//////////////////////////
    					OntPropertySubPropertyImplementation.replaceTag("<@propertynameuppercase@>", propName.toUpperCase());
    					OntPropertySubPropertyImplementation.replaceTag("<@propertyuppercase@>", sup.getLocalName().toUpperCase());
/////////////////////////		
    					
    					OntImplementation.insertIntoPlaceholder("<%subproperties%>", OntPropertySubPropertyImplementation.getString());
    					OntPropertySubPropertyImplementation.resetToInitValue();   
    					//break;
    				}
        			
    			}
    			
    			OntImplementation.insertIntoPlaceholder("<%properties%>", OntPropertyImplementation.getString());
    			OntPropertyImplementation.resetToInitValue();
    			
    		}	
    	}
    	
    	
    	itr = ontModel.listNamedClasses();
    	while(itr.hasNext()){    		
    		OntClass oc = (OntClass) itr.next();
    		String className = oc.getLocalName();
    		
    	    //ExtendedIterator restrictions = oc.li
    	    
    	    
    		OntClassDefinition.replaceTag("<@classname@>", className);
//////////////////
    		OntClassDefinition.replaceTag("<@classnameuppercase@>", className.toUpperCase());    		
//////////////////    		
			OntClassDefinition.replaceTag("<@prefix@>", prefix);
			
			OntDefinitionCFile.insertIntoPlaceholder("<%classes%>", OntClassDefinition.getString());
            OntImplementation.insertIntoPlaceholder("<%classesdefs%>", OntClassDefinition.getString());


//////
            OntClassDefinitionExtern.replaceTag("<@classname@>", className);
    		OntClassDefinitionExtern.replaceTag("<@classnameuppercase@>", className.toUpperCase());
			OntClassDefinitionExtern.replaceTag("<@prefix@>", prefix);

			OntDefinition.insertIntoPlaceholder("<%classes%>", OntClassDefinitionExtern.getString());


//////////////////
    			OntClassDefine.replaceTag("<@classnameuppercase@>", className.toUpperCase() + " 1");    
    			OntDefinition.insertIntoPlaceholder("<%includeontentities%>", OntClassDefine.getString());
       			OntClassDefine.resetToInitValue();
        
        		OntClassDefine.replaceTag("<@classnameuppercase@>", className.toUpperCase() + " 0");
    			OntDefinition.insertIntoPlaceholder("<%excludeontentities%>", OntClassDefine.getString() );
                OntClassDefine.resetToInitValue();
//////////////////
			
			
			OntClassDefinition.resetToInitValue();
            OntClassDefinitionExtern.resetToInitValue();

			
//////////////////
            OntClassImplementation.replaceAll("<@classnameuppercase@>", className.toUpperCase());
//////////////////    		
    		
    		OntClassImplementation.replaceAll("<@classname@>", className);
    		OntClassImplementation.replaceAll("<@prefix@>", prefix);
    		OntClassImplementation.replaceAll("<@classtype@>", oc.getURI());
    		ExtendedIterator pitr = oc.listDeclaredProperties();
    		while (pitr.hasNext()){
    			OntProperty op = (OntProperty) pitr.next();
    			OntClassPropertiesImplementation.replaceAll("<@prefix@>", prefix);
    			OntClassPropertiesImplementation.replaceAll("<@propertyname@>", op.getLocalName());
//////////////////
    			OntClassPropertiesImplementation.replaceAll("<@propertynameuppercase@>", op.getLocalName().toUpperCase());
//////////////////    			
    			OntClassPropertiesImplementation.replaceAll("<@classname@>", className);
    			OntClassPropertiesImplementation.replaceAll("<@classnameuppercase@>", className.toUpperCase());    			
    			OntClassImplementation.insertIntoPlaceholder("<%properties%>", OntClassPropertiesImplementation.getString());
    			OntClassPropertiesImplementation.resetToInitValue();
    		}
    		
    		
    		ExtendedIterator sitr = oc.listSuperClasses(true);
    		while(sitr.hasNext()){
    			OntClass sup = (OntClass) sitr.next();
    			OntClassSuperclassImplementation.replaceAll("<@prefix@>", prefix);
    			OntClassSuperclassImplementation.replaceAll("<@classname@>", className);
    			OntClassSuperclassImplementation.replaceAll("<@superclassname@>", sup.getLocalName());

//////////////////    			
    			OntClassSuperclassImplementation.replaceAll("<@classnameuppercase@>", className.toUpperCase());
    			OntClassSuperclassImplementation.replaceAll("<@superclassnameuppercase@>", sup.getLocalName().toUpperCase());
//////////////////
    			
    			OntImplementation.insertIntoPlaceholder("<%superclasses%>", OntClassSuperclassImplementation.getString());
    			OntClassSuperclassImplementation.resetToInitValue();    		
        		
    			
    		}
    		
    		OntImplementation.insertIntoPlaceholder("<%classes%>", OntClassImplementation.getString());
    		OntClassImplementation.resetToInitValue();    		
    		
    		
    	}
    	
    	LinkedList<LinkedList<String>> triples = getTripleOntology(ontModel);
	    for (LinkedList<String> list : triples) {
	    	String subject = list.get(0);
	    	String predicate = list.get(1);
	    	String object = list.get(2).replace("\n", "\\n");
//	    	OntImplementationOntologyTriple.replaceAll2("<@subject@>", subject);
//	    	OntImplementationOntologyTriple.replaceAll2("<@predicate@>", predicate);
//	    	OntImplementationOntologyTriple.replaceAll2("<@object@>", object);	    	  	
	    	
//	    	OntImplementation.insertIntoPlaceholder("<%ontologytriples%>", OntImplementationOntologyTriple.getString());
//	    	OntImplementationOntologyTriple.resetToInitValue();
	   
	    }
    	

         //OntDefinitionCFile.removeTags();
    //    writer.addCDef(OntDefinitionCFile.getString());

    	OntImplementation.removeTags();
        writer.addImpl(OntImplementation.getString());
    	
        
        OntDefinition.removeTags();
        writer.addDef(OntDefinition.getString());

       //OntDefinitionCFile.removeTags();
       //writer.addCDef(OntDefinitionCFile.getString());
    }

}
