package ru.karelia.cs.petrsugen.handlers.cHandlers;

import java.util.ArrayList;
import java.util.Iterator;



import ru.karelia.cs.petrsugen.templates.TemplateResourceFile;

import com.hp.hpl.jena.ontology.OntClass;
import com.hp.hpl.jena.ontology.OntProperty;
import com.hp.hpl.jena.util.iterator.ExtendedIterator;

import ru.karelia.cs.petrsugen.framework.CCodeModelContainer;
import ru.karelia.cs.petrsugen.util.CodeGenerationUtils;
import ru.karelia.cs.petrsugen.util.TemplateString;

/**
 *
 * 
 */
public class CClassHandler {

    static String definitionTemplateName = TemplateResourceFile.getTemplate("CClassDefinition");
    static String declTemplateName = TemplateResourceFile.getTemplate("CDeclaration");
    static String createClassTemplateName = TemplateResourceFile.getTemplate("CClassImplementation");
    static String CClassGetCallTemplateName = TemplateResourceFile.getTemplate("CClassGetCall");
    static String CClassFreeDPCallTemplateName = TemplateResourceFile.getTemplate("CClassFreeDPCall");
    static String CClassFreeOPCallTemplateName = TemplateResourceFile.getTemplate("CClassFreeOPCall");
    
    
    
    public static void handle(OntClass ontClass, CCodeModelContainer writer) {
        ExtendedIterator itr = ontClass.listDeclaredProperties();
        String className = ontClass.getLocalName().toLowerCase();
        ArrayList<OntProperty> dps = new ArrayList<OntProperty>();
        ArrayList<OntProperty> ops = new ArrayList<OntProperty>();
        
        System.out.println("CLASS : "  + className + "  NAMESPACE: " + ontClass.getNameSpace());
        while (itr.hasNext()) {
            Object object = itr.next();

            OntProperty op = (OntProperty) object;

            if (op.isDatatypeProperty()) {
            	// this is a DIRTY HACK, if there is a uuid dp in the ontology we  skip it!!!
    			if (!op.getLocalName().equalsIgnoreCase("uuid")){
                    dps.add(op);
    			}
            }
            if (op.isObjectProperty()) {

                ops.add(op);

            }
        }

        /* generates the typedef struct for the class */

        TemplateString classTypedef = new TemplateString(definitionTemplateName);
        TemplateString classDecl = new TemplateString(declTemplateName);
        classTypedef.replaceTag("<@classname@>", className);
        // adds datatypepropertys to the struct  
        Iterator dpitr = dps.iterator();

        while (dpitr.hasNext()) {

            OntProperty dp = (OntProperty) dpitr.next();
     
            classDecl.replaceAll("<@declarationtype@>", "	dp_GSList *");
            classDecl.replaceAll("<@declarationname@>", CodeGenerationUtils.removeUri(dp.getLocalName()).toLowerCase());
            classDecl.replaceAll("<%tabs%>", "\t");

            classTypedef.insertIntoPlaceholder("<%declarations%>", classDecl.getString());

            classDecl.resetToInitValue();

        }
        
        // adds objectpropertys to the struct

        Iterator opitr = ops.iterator();

        while (opitr.hasNext()) {
            OntProperty op = (OntProperty) opitr.next();

            classDecl.replaceAll("<@declarationtype@>", "	op_GSList *");
            classDecl.replaceAll("<@declarationname@>", CodeGenerationUtils.removeUri(op.getLocalName()).toLowerCase());
            classDecl.replaceAll("<%tabs%>", "\t");
            classTypedef.insertIntoPlaceholder("<%declarations%>", classDecl.getString());
            classDecl.resetToInitValue();

        }

        classTypedef.replaceTag("<@headerfile@>", writer.getHeaderFile());
        classTypedef.removeTags();
        writer.addDef(classTypedef.getString());

        /* end of typedef generation */

        String classNameReal = ontClass.getNameSpace() + ontClass.getLocalName();
        TemplateString createClass = new TemplateString(createClassTemplateName);
        createClass.replaceAll("<@classname@>", className);
        createClass.replaceAll("<@classnamereal@>", classNameReal);
        createClass.replaceAll("<@namespace@>", ontClass.getNameSpace().toString());
        
        createClass.replaceAll("<@classtype@>", ontClass.getURI());
        TemplateString classGet = new TemplateString(CClassGetCallTemplateName);
       
        
        TemplateString classFreeDPCall = new TemplateString(CClassFreeDPCallTemplateName);
        TemplateString classFreeOPCall = new TemplateString(CClassFreeOPCallTemplateName);
        
        dpitr = dps.iterator();
  
        // to generate the function that populates this struct from the SIB
        while (dpitr.hasNext()) {
            
        	OntProperty dp = (OntProperty) dpitr.next();
            classGet.replaceAll("<@classname@>", className);
                        
            String dpName = CodeGenerationUtils.removeUri(dp.getLocalName()).toLowerCase();
            createClass.insertIntoPlaceholder("<%propertylist%>","	individual->"+dpName +"= NULL; \n");
            classGet.replaceAll("<@propertyname@>", dpName);
            
            createClass.insertIntoPlaceholder("<%getcalls%>", classGet.getString());
            classGet.resetToInitValue();
            classFreeDPCall.replaceAll("<@propertyname@>", dpName);
            classFreeDPCall.replaceAll("<@classname@>", className);
            createClass.insertIntoPlaceholder("<%freecalls%>", classFreeDPCall.getString());
            classFreeDPCall.resetToInitValue();
        }

        opitr = ops.iterator();
        
        // add objectproperties to the function that populets this struct from the SIB
        while (opitr.hasNext()) {
            OntProperty op = (OntProperty) opitr.next();
            classGet.replaceAll("<@classname@>", className);
           
            String opName = CodeGenerationUtils.removeUri(op.getLocalName()).toLowerCase();
            classGet.replaceAll("<@propertyname@>", opName);
            
            createClass.insertIntoPlaceholder("<%propertylist%>","individual->"+opName +"= NULL; \n");
            createClass.insertIntoPlaceholder("<%getcalls%>", classGet.getString());
            classGet.resetToInitValue();
                        
            classFreeOPCall.replaceAll("<@propertyname@>", opName);
            classFreeOPCall.replaceAll("<@classname@>", className);
            createClass.insertIntoPlaceholder("<%freecalls%>", classFreeOPCall.getString());
            classFreeOPCall.resetToInitValue();
        }

        createClass.removeTags();
        writer.addImpl(createClass.getString());

       
        dpitr = dps.iterator();
       
        while (dpitr.hasNext()) {
            OntProperty dp = (OntProperty) dpitr.next();
            CDatatypePropertyHandler.handle(dp, writer, className);

        }
        opitr = ops.iterator();

        while (opitr.hasNext()) {
            OntProperty op = (OntProperty) opitr.next();
            CObjectPropertyHandler.handle(op, writer, className);
        }

    }
}
