package ru.karelia.cs.petrsugen.handlers.cHandlers;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;



import ru.karelia.cs.petrsugen.templates.TemplateResourceFile;

import com.hp.hpl.jena.graph.Triple;
import com.hp.hpl.jena.ontology.CardinalityRestriction;
import com.hp.hpl.jena.ontology.OntClass;
import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.ontology.OntProperty;
import com.hp.hpl.jena.ontology.OntResource;
import com.hp.hpl.jena.ontology.Restriction;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.rdf.model.StmtIterator;
import com.hp.hpl.jena.util.iterator.ExtendedIterator;

import ru.karelia.cs.petrsugen.framework.CCodeModelContainer;
import ru.karelia.cs.petrsugen.util.CodeGenerationUtils;
import ru.karelia.cs.petrsugen.util.Logger;
import ru.karelia.cs.petrsugen.util.TemplateString;

/**
 *
 * 
 */
public class OntHandler {

	
	static String OntImplementationTemplateName = TemplateResourceFile.getTemplate("OntImplementation");
	static String OntClassImplementationTemplateName = TemplateResourceFile.getTemplate("OntClassImplementation");
	static String OntPropertyImplementationTemplateName = TemplateResourceFile.getTemplate("OntPropertyImplementation");
	static String OntClassSuperclassImplementationTemplateName = TemplateResourceFile.getTemplate("OntClassSuperclassImplementation");
	static String OntClassPropertiesImplementationTemplateName = TemplateResourceFile.getTemplate("OntClassPropertiesImplementation");
	static String OntPropertySubPropertyImplementationTemplateName =  TemplateResourceFile.getTemplate("OntPropertySubPropertyImplementation");
	
	static String  OntDefinitionTemplateName = TemplateResourceFile.getTemplate("OntDefinition");
	static String  OntClassDefinitionTemplateName = TemplateResourceFile.getTemplate("OntClassDefinition");
	static String  OntPropertyDefinitionTemplateName = TemplateResourceFile.getTemplate("OntPropertyDefinition");
	
	static String OntImplementationOntologyTripleName = TemplateResourceFile.getTemplate("OntImplementationOntologyTriple");
	public static LinkedList<LinkedList<String>> getTripleOntology(OntModel model) {
	    	LinkedList<LinkedList<String>> triples = new LinkedList<LinkedList<String>>();
	    	if (model != null) {
	            /*
	            this.classes = new LinkedList<ClassTree>();
	            this.deserializeModel(model);
	             */
	            StmtIterator iterator = model.listStatements();
	            LinkedList<String> tripleList;
	            Triple triple = null;
	            int cntStatements = 0;
	            int cntStatementsInBaseModel = 0;
	            while (iterator.hasNext()) {
	                tripleList = new LinkedList<String>();
	                Statement statement = iterator.nextStatement();
	                cntStatements++;
	                if ( model.isInBaseModel(statement) ) {
	                    //System.out.println("Yes "+cntStatementsInBaseModel);
	                    cntStatementsInBaseModel++;
	                    triple = statement.asTriple();

	                    tripleList.add(triple.getSubject().toString());
	                    tripleList.add(triple.getPredicate().toString());
	                    tripleList.add(triple.getObject().toString());
	                    if (triple.getObject().isLiteral()) {
	                        tripleList.add("True");
	                    } else if (triple.getObject().isURI()) {
	                        tripleList.add("False");
	                    } else if (triple.getObject().isConcrete()) {
	                        tripleList.add("False");
	                    } else if (triple.getObject().isVariable()) {
	                        tripleList.add("False");
	                    } else if (triple.getObject().isBlank()) {
	                        tripleList.add("False");
	                    }
	                    triples.add(tripleList);
	                } else {
	                    //System.out.println("No "+ (cntStatements-cntStatementsInBaseModel ));
	                }
	            }
	            //System.out.println("Total statements (assertions): " + cntStatements);
	            //System.out.println("Statements in base model (assertions): " + cntStatements);
	        }
	    
	    	return triples;
	    }
	
    
    public static void handle(OntModel ontModel, CCodeModelContainer writer, Logger log) {
      
        TemplateString OntImplementation = new TemplateString(OntImplementationTemplateName);
        TemplateString OntClassImplementation = new TemplateString(OntClassImplementationTemplateName);
        TemplateString OntPropertyImplementation = new TemplateString(OntPropertyImplementationTemplateName);
    	TemplateString OntPropertySubPropertyImplementation = new TemplateString(OntPropertySubPropertyImplementationTemplateName);
        TemplateString OntClassPropertiesImplementation = new TemplateString(OntClassPropertiesImplementationTemplateName);
        TemplateString OntClassSuperclassImplementation = new TemplateString(OntClassSuperclassImplementationTemplateName);
    	
        TemplateString OntDefinition = new TemplateString(OntDefinitionTemplateName);
        TemplateString OntClassDefinition = new TemplateString(OntClassDefinitionTemplateName);
        TemplateString OntPropertyDefinition = new TemplateString(OntPropertyDefinitionTemplateName);
        
		TemplateString OntImplementationOntologyTriple = new TemplateString(OntImplementationOntologyTripleName);		
        
        
    	String prefix="";
    	
    	OntImplementation.replaceTag("<@headerfile@>", writer.getHeaderFile());	
    	
    	ExtendedIterator itr = ontModel.listOntProperties();
    	
    	while(itr.hasNext()){
    		OntProperty op = (OntProperty) itr.next();
    		int maxcardinality = 0;
    		int mincardinality = 0;
    		System.out.println("OP: " + op.getURI());
    		// functionalpropertys are mapped to maxcardinality
			if (op.isFunctionalProperty()){
    			System.out.println("Functional property "+ op.getLocalName());        			
    			maxcardinality = 1;
    		}
			
    		
    		if (op.isDatatypeProperty() || op.isObjectProperty()){
    				
                ExtendedIterator restrictions = op.listReferringRestrictions();
                while(restrictions.hasNext()){
                	
                	Restriction res = (Restriction) restrictions.next();
                	if (res.isCardinalityRestriction()){
                        CardinalityRestriction cardinality = res.asCardinalityRestriction();
                        if (cardinality.isMaxCardinalityRestriction())
                        	mincardinality = cardinality.getCardinality();
                        
                        if (cardinality.isMaxCardinalityRestriction())
                        	maxcardinality = cardinality.getCardinality();
                        
                        System.out.print("Cardinaliy ");
                	}
                	//System.out.println("Restriction name " + res.getLocalName());
                }
    			
    			String propName = op.getLocalName();
    			
    			OntPropertyDefinition.replaceTag("<@propertyname@>", propName);
    			OntPropertyDefinition.replaceTag("<@prefix@>", prefix);    			
    			OntDefinition.insertIntoPlaceholder("<%properties%>", OntPropertyDefinition.getString());
    			OntPropertyDefinition.resetToInitValue();
    			
    			
    			OntPropertyImplementation.replaceTag("<@propertyname@>", propName);
    			OntPropertyImplementation.replaceTag("<@prefix@>", prefix);
    			OntPropertyImplementation.replaceTag("<@maxcardinality@>", Integer.toString(maxcardinality));
    			OntPropertyImplementation.replaceTag("<@mincardinality@>", Integer.toString(mincardinality ));
    			OntPropertyImplementation.replaceTag("<@name@>", propName);
    			
    			OntPropertyImplementation.replaceTag("<@about@>", op.getURI().toString());
    			if (op.getRange()!= null){
    				//OntPropertyImplementation.replaceTag("<@range@>", op.getRange().toString());
    				
    				OntPropertyImplementation.replaceTag("<@range@>", op.getRange().toString());
    				System.out.println("Teh range "+op.getRange());
    			} else {
    				System.out.println(" OP has null Range: "+ propName);    			
    			}
    			if (op.getDomain()!= null){
    				OntPropertyImplementation.replaceTag("<@domain@>", op.getDomain().toString());
    			} else {
    				System.out.println(" OP has null Domain: "+ propName);
    			
    			}
    			if (op.isInverseFunctionalProperty()) {
    				OntPropertyImplementation.replaceTag("<@inversfunctionalproperty@>", "\""+op.getRange().toString()+"\"");	
       			
        		} else {
        			OntPropertyImplementation.replaceTag("<@inversfunctionalproperty@>", "NULL");
        			
        		}      		
    			
    			if (op.isObjectProperty()){
    				
    				OntPropertyImplementation.replaceTag("<@type@>", "OBJECTPROPERTY");
    				OntPropertyImplementation.replaceTag("<@set@>", "op_property_set");
    				OntPropertyImplementation.replaceTag("<@xsdtype@>", "NONE");

    			} else {
    				OntPropertyImplementation.replaceTag("<@type@>", "DATATYPEPROPERTY");
    				OntPropertyImplementation.replaceTag("<@set@>", "dp_property_set");
    				
    				// HACK HACK. FIX FIX
    				if (op.getRange() != null){
    					String dpType = CodeGenerationUtils.removeUri(op.getRange().toString());
    					OntPropertyImplementation.replaceTag("<@xsdtype@>", "XSD"+dpType.toUpperCase());
    				} else {
    					System.out.println(" HACK STUFF ");
    					OntPropertyImplementation.replaceTag("<@xsdtype@>", "XSDSTRING");
    					log.warning("NULL xsdstring found, replaced with XSDSTRING");
    				
    				}
    				/*if (op.getRange()!= null){
    					String dpType = CodeGenerationUtils.removeUri(op.getRange().toString());
    					OntPropertyImplementation.replaceTag("<@xsdtype@>", "XSD"+dpType.toUpperCase());
    				} else{
    					//og.warning("Generated DP)
    					//OntPropertyImplementation.replaceTag("<@xsdtype@>", "XSD"+dpType.toUpperCase());
    				}*/
    				
    			}
    			ExtendedIterator sitr = op.listSuperProperties();
    			while(sitr.hasNext()){
    				
    				OntProperty sup = (OntProperty) sitr.next();
    				if (!sup.getLocalName().equals(op.getLocalName())){
    					OntPropertySubPropertyImplementation.replaceTag("<@prefix@>", prefix);
    					OntPropertySubPropertyImplementation.replaceTag("<@propertyname@>", propName);
    					OntPropertySubPropertyImplementation.replaceTag("<@property@>", sup.getLocalName());
    					OntImplementation.insertIntoPlaceholder("<%subproperties%>", OntPropertySubPropertyImplementation.getString());
    					OntPropertySubPropertyImplementation.resetToInitValue();   
    					//break;
    				}
        			
    			}
    			
    			OntImplementation.insertIntoPlaceholder("<%properties%>", OntPropertyImplementation.getString());
    			OntPropertyImplementation.resetToInitValue();
    			
    		}	
    	}
    	
    	
    	itr = ontModel.listNamedClasses();
    	while(itr.hasNext()){    		
    		OntClass oc = (OntClass) itr.next();
    		String className = oc.getLocalName();
    		
    	    //ExtendedIterator restrictions = oc.li
    	    
    	    
    		OntClassDefinition.replaceTag("<@classname@>", className);
			OntClassDefinition.replaceTag("<@prefix@>", prefix);
			
			OntDefinition.insertIntoPlaceholder("<%classes%>", OntClassDefinition.getString());
			OntClassDefinition.resetToInitValue();
			
    		
    		OntClassImplementation.replaceAll("<@classname@>", className);
    		OntClassImplementation.replaceAll("<@prefix@>", prefix);
    		OntClassImplementation.replaceAll("<@classtype@>", oc.getURI());
    		ExtendedIterator pitr = oc.listDeclaredProperties();
    		while (pitr.hasNext()){
    			OntProperty op = (OntProperty) pitr.next();
    			OntClassPropertiesImplementation.replaceAll("<@prefix@>", prefix);
    			OntClassPropertiesImplementation.replaceAll("<@propertyname@>", op.getLocalName());
    			OntClassPropertiesImplementation.replaceAll("<@classname@>", className);
    			OntClassImplementation.insertIntoPlaceholder("<%properties%>", OntClassPropertiesImplementation.getString());
    			OntClassPropertiesImplementation.resetToInitValue();
    		}    		
    		
    		ExtendedIterator sitr = oc.listSuperClasses(true);
    		while(sitr.hasNext()){
    			OntClass sup = (OntClass) sitr.next();
    			OntClassSuperclassImplementation.replaceAll("<@prefix@>", prefix);
    			OntClassSuperclassImplementation.replaceAll("<@classname@>", className);
    			OntClassSuperclassImplementation.replaceAll("<@superclassname@>", sup.getLocalName());
    			OntImplementation.insertIntoPlaceholder("<%superclasses%>", OntClassSuperclassImplementation.getString());
    			OntClassSuperclassImplementation.resetToInitValue();    		
        		
    			
    		}
    		
    		OntImplementation.insertIntoPlaceholder("<%classes%>", OntClassImplementation.getString());
    		OntClassImplementation.resetToInitValue();    		
    		
    		
    	}
    	
    	LinkedList<LinkedList<String>> triples = getTripleOntology(ontModel);
	    for (LinkedList<String> list : triples) {
	    	String subject = list.get(0);
	    	String predicate = list.get(1);
	    	String object = list.get(2).replace("\n", "\\n");
	    	OntImplementationOntologyTriple.replaceAll2("<@subject@>", subject);
	    	OntImplementationOntologyTriple.replaceAll2("<@predicate@>", predicate);
	    	OntImplementationOntologyTriple.replaceAll2("<@object@>", object);	    	  	
	    	
	    	OntImplementation.insertIntoPlaceholder("<%ontologytriples%>", OntImplementationOntologyTriple.getString());
	    	OntImplementationOntologyTriple.resetToInitValue();
	   
	    }
    	
    	
    	OntImplementation.removeTags();
        writer.addImpl(OntImplementation.getString());
    	
        
        OntDefinition.removeTags();
        writer.addDef(OntDefinition.getString());
    }

}
