package ru.karelia.cs.petrsugen.util;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.Iterator;
import java.util.LinkedList;

import com.hp.hpl.jena.ontology.OntClass;
import com.hp.hpl.jena.ontology.OntProperty;
import com.hp.hpl.jena.util.iterator.ExtendedIterator;

public class CodeGenerationUtils {

    public static LinkedList<String> filter = new LinkedList<String>();

    public static String toVariableName(String string) {
        string = CodeGenerationUtils.removeUri(string);
        if (string.startsWith("has")) {
            String[] split = string.split("has");
            if(split.length == 2) {
                string = split[1];
            } else {
                
            }
        }
        string = string.substring(0, 1).toLowerCase() + string.substring(1);
        return string;
    }

    public static String toFunctionName(String string) {
        string = CodeGenerationUtils.removeUri(string);
        if (string.startsWith("has")) {
            String[] split = string.split("has");
            if(split.length == 2) {
                string = split[1];
            } else {
                
            }
        }
        string = string.substring(0, 1).toUpperCase() + string.substring(1);
        return string;
    }

    public static String removeUri(String uri) {
        if (uri != null) {
            String elements[] = uri.split("#");
            String element = uri;
            if (elements.length > 1) {
                element = elements[1];
                element.trim();
            }
            return element;
        } else {
            return null;
        }
    }

    public static String extractFileName(String absoluteFileName) {
        if (absoluteFileName != null) {
            absoluteFileName = absoluteFileName.substring(absoluteFileName.lastIndexOf(File.separatorChar) + 1, absoluteFileName.lastIndexOf("."));
        }
        return absoluteFileName;
    }

    public static StringBuilder readTemplate(
            String fileName) {

        StringBuilder contents = new StringBuilder();
        String line;

        BufferedReader templateFile;

        try {
            templateFile = new BufferedReader(new FileReader(fileName));

            while ((line = templateFile.readLine()) != null) {
                contents.append(line + "\n");
            }

            templateFile.close();
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }

        return contents;
    }

    public static String capitalizeFirstLetter(String input) {
        String retVal = null;
        if (input != null) {
            String firstLetter = input.substring(0, 1);
            String remainder = input.substring(1);
            retVal = firstLetter.toUpperCase() + remainder;
        }
        return retVal;
    }

    public static LinkedList<OntClass> filterClasses(ExtendedIterator classes) {
        LinkedList<OntClass> retVal = new LinkedList<OntClass>();

        while (classes.hasNext()) {
            OntClass ontClass = (OntClass) classes.next();
            String className = ontClass.getLocalName();
            if (className != null) {
                className.trim();

                if (!filter.contains(className)) {
                    retVal.add(ontClass);
                }
            }
        }
        return retVal;
    }

    public static void populateFilter() {
        filter.add("Thing");
        filter.add("Resource");
        filter.add("List");
        filter.add("Literal");
        filter.add("Property");
        filter.add("Ontology");
        filter.add("Property");
        filter.add("Statement");
        filter.add("Restriction");
        filter.add("Nothing");
        filter.add("Class");
        filter.add("decimal");
        filter.add("integer");
        filter.add("int");
        filter.add("long");
        filter.add("short");
        filter.add("byte");
        filter.add("unsignedShort");
        filter.add("unsignedLong");
        filter.add("float");
        filter.add("dateTime");
        filter.add("string");
        filter.add("unsignedInt");
        filter.add("nonNegativeInteger");
        filter.add("duration");
        filter.add("unsignedByte");
        filter.add("date");
        filter.add("boolean");
        filter.add("nonPositiveInteger");
        filter.add("time");
    }

    public static boolean checkForValidity(OntClass ontClass) {
        boolean retVal = true;
        Iterator itr = filter.iterator();
        while (itr.hasNext()) {
            String filterName = (String) itr.next();
            if (ontClass.getLocalName().equalsIgnoreCase(filterName)) {
                retVal = false;
                break;
            }
        }
        return retVal;
    }

    public static String toRange(String range) {
        String retVal = "";
        Iterator itr = CodeGenerationUtils.filter.iterator();
        while (itr.hasNext()) {
            String next = (String) itr.next();
            if (next.equalsIgnoreCase(range)) {
                range = "XSD" + CodeGenerationUtils.capitalizeFirstLetter(range);
                break;
            }
        }
        retVal = range;
        return retVal;
    }

    public static void getOrderedClassHierarchy(OntClass thing, LinkedList list) {

        //System.out.println("current class: " + thing.getLocalName());
        if (thing != null) {
            if (!list.contains(thing)) {
                list.add(thing);
            }
            if (thing.listSubClasses() != null) {
                Iterator itr = thing.listSubClasses();
                while (itr.hasNext()) {
                    OntClass subClass = (OntClass) itr.next();
                    //System.out.println("current subclass: " + subClass.getLocalName());
                    if (checkForValidity(subClass)) {
                        getOrderedClassHierarchy(subClass, list);
                    }
                }
            }
        }
    }

    public static void getOrderedPropertyClassHierarchy(OntProperty property, LinkedList list) {
        if (property != null) {
            if (!list.contains(property)) {
                list.add(property);
            }
            if (property.listSubProperties(true) != null) {
                Iterator itr = property.listSubProperties(true);
                while (itr.hasNext()) {
                    OntProperty subProperty = (OntProperty) itr.next();
                    if (!list.contains(subProperty)) {
                        getOrderedPropertyClassHierarchy(subProperty, list);
                    }
                }
            }
        }
    }

    public static <T> void removeDuplicates(LinkedList<T> aList) {
        T currValue;
        T target;
        Iterator<T> curr;
        curr = aList.iterator();
        target = curr.next();
        while (curr.hasNext()) {
            currValue = curr.next();
            if (currValue.equals(target)) {
                curr.remove();
            } else {
                target = currValue;
            }
        }
    }

    // a bit unsure about this one... :)
    public static LinkedList<OntClass> removeDuplicatesFromOntClass(LinkedList<OntClass> aList) {
        String currValueName;
        for (OntClass ontClass : aList) {
            System.out.println("the ontology: " + ontClass.getLocalName());
        }
        String targetValueName;
        LinkedList<OntClass> tmpList = (LinkedList<OntClass>)aList.clone();
        for(OntClass curr : aList) {
            for(OntClass target : aList) {
                if(curr == target) {
                    continue;
                }
                currValueName = curr.getLocalName();
                targetValueName = target.getLocalName();
                if (currValueName.equalsIgnoreCase(targetValueName)) {
                    tmpList.remove(target);
                    System.out.println("removed duplicate: " + currValueName);
                }
            }
        }
        for (OntClass ontClass : tmpList) {
            System.out.println("the ontology tmp: " + ontClass.getLocalName());
        }
        return tmpList;
    }
}
